/* /////////////////////////////////////////////////////////////////////////////////////////////////

Author: PluginIdea
Email:sugamer8 @gmail.com
 Patreon:https://www.patreon.com/PluginIdea

©2022 PluginIdea, Inc.All rights reserved.

///////////////////////////////////////////////////////////////////////////////////////////////// */

using UnityEngine;using System.Collections.Generic;using MeshVR;using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;using System.Linq;using Battlehub.RTCommon;namespace PluginIdea{    public class GizmosSystem : MVRScript, IGL    {        public const string InputModule = "InputModule";        public const string HudHanderModule = "HudHanderModule";        public const string MeshDrawModule = "MeshDrawModule";        public const string LightDrawModule = "LightDrawModule";        public const string CameraDrawModule = "CameraDrawModule";        public const string PersonDrawModule = "PersonDrawModule";        public const string AudioSourceDrawModule = "AudioSourceDrawModule";        public const string FreeControllerModule = "FreeControllerModule";        public const string ColliderDrawModule = "ColliderDrawModule";        private Dictionary<string, GZModuleBase> uid2Modules = new Dictionary<string, GZModuleBase>();        private Dictionary<string, Mesh> dic_Meshs = new Dictionary<string, Mesh>();        private List<string> meshnames = new List<string> { "arm", "chest", "enbow", "footL", "footR", "handL", "handR", "head", "knee", "pelvis" };        private Dictionary<string, Shader> dic_BundleShaders = new Dictionary<string, Shader>();        private Dictionary<string, Camera> dic_name2Camera = new Dictionary<string, Camera>();        private Atom currentSelectedAtom;        private Camera currentSelectedCamera;        private CameraHook cameraHook;        public Camera CurrentSelectedCamera        {            get            {                return currentSelectedCamera;            }        }        public Atom CurrentDrawAtom        {            get {                return this.currentSelectedAtom;            }        }        public Dictionary<string, Shader> Dic_BundleShaders        {            get {                return this.dic_BundleShaders;            }        }        public Dictionary<string, Mesh> Dic_Meshes        {            get {                return this.dic_Meshs;            }        }        public Transform CenterCameraTrans        {            get            {                if (Utils.IsInVR())                {                    if (SuperController.singleton.isOpenVR)                    {                        return SuperController.singleton.ViveCenterCamera.transform;                    }                    return SuperController.singleton.OVRCenterCamera.transform;                }                else                {                    return SuperController.singleton.MonitorCenterCamera.transform;                }            }        }        public override void Init()        {            base.Init();            Utils.SetupInfoText(this, "<size=40><b>GizmosSystem 1.1</b></size>\n\n" +                "A series of tool sets that assist the display and operation in the scene. Typical ones include: topology display, light audio range display, brand-new character controller Icon, etc. Each tool can be switched on and off independently..\n\n" +                 "<b>Topology:</b>The Mesh topology of the selected object is displayed. The related display parameters can be adjusted..\n\n" +                 "<b>Light:</b>Range display of several different lights\n\n" +                 "<b>AudioSource:</b>Range display of several different AudioSource\n\n" +                 "<b>Controller's Color&Icon:</b>Color-distinguish each controller in the scene according to Atom. It is convenient to distinguish each Atom. " +                 "At the same time, the controller icons of the characters are customized (head, hands, feet, knees, etc.)\n\n" +                 "<b>Controller's Color&Icon Parameter:</b>Refresh List : Refresh the list of characters in the scene. After selecting a task, you can set a custom color for the controller Icon of the specified character.\n\n" +                 "<b>Genneric Collider:</b>Draw the outline of some colliders in the scene..\n\n", 400, true);            ResourceInit();        }        private void DelayInit()        {            DataInit();            UIInit();            EventInit();            LogicInit();        }        private void ResourceInit()        {            string assetsBundlePath = Utils.GetPluginPath(this) + "/resources/gzsystem.assetbundle";            Request request = new Request { path = assetsBundlePath, callback = OnAssetBundleLoaded };            AssetLoader.QueueLoadAssetBundleFromFile(request);        }        private void OnAssetBundleLoaded(Request request)        {            Shader[] shaders = request.assetBundle.LoadAllAssets<Shader>();            for (int i = 0; i < shaders.Length; i++)            {                                if (!dic_BundleShaders.ContainsKey(shaders[i].name))                {                    dic_BundleShaders.Add(shaders[i].name, shaders[i]);                }            }            meshnames.ForEach(n => {                GameObject gb = request.assetBundle.LoadAsset<GameObject>(n);                Mesh mesh = gb.GetComponentInChildren<MeshFilter>().sharedMesh;                                dic_Meshs.Add(n, mesh);            });            DelayInit();        }                private void DataInit()        {        }        private void UIInit()        {            Utils.OnInitUI(CreateUIElement);        }        private void EventInit()        {            RegisterModule(InputModule, new InputModule(this));            RegisterModule(MeshDrawModule, new MeshDrawMoudle(this));            RegisterModule(LightDrawModule, new LightDrawModule(this));            RegisterModule(PersonDrawModule, new PersonDrawModule(this));            RegisterModule(AudioSourceDrawModule, new AudioSourceDrawModule(this));            RegisterModule(FreeControllerModule, new FreeControllerModule(this));            RegisterModule(ColliderDrawModule, new ColliderDrawModule(this));        }        private void LogicInit()        {        }        private void Start()        {            if (Utils.IsInVR())            {                cameraHook = Utils.GetSafeComponent<CameraHook>(CenterCameraTrans.gameObject);                cameraHook.onPostRenderCallback += OnPostRender;            }            else            {                if (GLRenderer.Instance == null)                {                    new GameObject                    {                        name = "GLRenderer"                    }.AddComponent<GLRenderer>();                }                GLRenderer.Instance.Add(this);            }        }                private void Update()        {            currentSelectedAtom = SuperController.singleton.GetSelectedAtom();                        foreach (var module in uid2Modules.Values.ToList())            {                module.OnUpdate();            }        }        private void RegisterModule(string key, GZModuleBase module)        {            if (!uid2Modules.ContainsKey(key))            {                uid2Modules.Add(key, module);                module.Init();            }        }        private void UnRegisterModule(string key)        {            if (uid2Modules.ContainsKey(key))            {                uid2Modules[key].UnInit();                uid2Modules.Remove(key);            }        }        private void OnDestroy()        {            if (uid2Modules.Count > 0)            {                foreach (var key in uid2Modules.Keys.ToList())                {                    UnRegisterModule(key);                }                uid2Modules.Clear();            }            if (Utils.IsInVR())            {                Destroy(cameraHook);            }            else            {                GLRenderer.Instance.Remove(this);            }            Utils.OnDestroyUI();        }        private void OnPostRender()        {            foreach (var module in uid2Modules.Values.ToList())            {                module.OnPostRender();            }        }        public void Draw(int cullingMask)        {            OnPostRender();        }    }}